/*

ea_plt1003_usb_mouse.c

Mouse demo based on the EmbeddedAdventures PLT-1003 USB to Serial platform.

This demo was originally based on the TechToys 18F4550 board, which has a four-way
joystick plus press-button. 

Since the PLT-1003 only has two buttons, we just make the mouse cursor go left and right.

*/


// System includes
#include "string.h"

// Config
#include "config.h"


// General PIC utilities
#include "pic_utils.h"

// Timer 0 functions
#include "pic_timer.h"

// Tick functions (builds on timer functions)
#include "pic_tick.h"

// Serial port communications
#include "pic_serial.h"

// Terminal functions (builds on serial port)
#include "pic_term.h"

// Platform LEDs
#include "platform_leds.h"

// USB general routines
#include "pic_usb.h"


uns16	tick_marker = 0;

uns8	up_count,
		down_count,
		right_count,
		left_count,
		center_count,
		check_count;

signed char buffer[3];

// Interrupt routine - - - - - - - - - -

void interrupt() {
	
		serial_handle_tx_isr();
		serial_handle_rx_isr();
		usb_handle_isr();	// USB priority isn't so important in this demo
		timer_handle_0_isr();
}


// configure_system

void configure_system() {
	
	// Turn off interupts just in case
	kill_interrupts();

	// Kill those pesky analogue inputs
	turn_analog_inputs_off();
	
	// Set serial ready to go at 115k	
	serial_setup(SPBRG_115200);

	// Use 0xffff - 4000 if you're using techtoys board at 16Mhz (1ms)
	// PLT-1003 runs at 48Mhz so we need bigger number
	timer_setup_0(TIMER_16BIT_MODE, TIMER_PRESCALER_OFF, 0xffff - 12000); 
	
	// Initialise the terminal library
	term_init();
	
	// Configure our pins for inputs
	#ifdef UP_PIN
		make_input(JOY_PORT, UP_PIN); 
	#endif
	#ifdef DOWN_PIN
		make_input(JOY_PORT, DOWN_PIN);
	#endif
	#ifdef LEFT_PIN	
		make_input(JOY_PORT, LEFT_PIN); 
	#endif
	#ifdef RIGHT_PIN	
		make_input(JOY_PORT, RIGHT_PIN);
	#endif
	#ifdef CENTER_PIN
		make_input(JOY_PORT, CENTER_PIN); 
	#endif
	
	#ifdef NEED_WEAK_PULLUPS
		clear_bit(intcon2, RBPU); // turn on weak pull-ups
	#endif
	
	// Get USB subsystem ready
	usb_setup();

	// Turn on peripheral interrupts
	turn_peripheral_ints_on();
	
	// Turn on USB interrupts 
	turn_usb_ints_on();
	
	// Turn on all interrupts
	turn_global_ints_on();
}



void term_entry_callback(uns8 *term_buffer) {

uns8 data[5];
uns8 first_char;
	
	first_char = term_buffer[0];

	switch (first_char) {
						}
}						


void mouse_process() {

uns16 test_tick;

	test_tick = tick_get_count();	// find out what we're up to
	if (tick_calc_diff(tick_marker, test_tick) > 3) {	// every 3 ms
		tick_marker = test_tick;
		#ifdef UP_PIN
			if (!test_pin(JOY_PORT, UP_PIN)) {
				up_count++;
			}
		#endif
		#ifdef DOWN_PIN		
			if (!test_pin(JOY_PORT, DOWN_PIN)) {
				down_count++;
			}
		#endif
		#ifdef LEFT_PIN	
			if (!test_pin(JOY_PORT, LEFT_PIN)) {
				left_count++;
			}
		#endif
		#ifdef RIGHT_PIN		
			if (!test_pin(JOY_PORT, RIGHT_PIN)) {
				right_count++;
			}	
		#endif
		#ifdef CENTER_PIN
			if (!test_pin(JOY_PORT, CENTER_PIN)) {
				center_count++;
			}
		#endif	
		check_count++;
		if (check_count == 5) {
			check_count = 0;
			if (usb_get_state() == st_CONFIGURED) {
				
				if (center_count) {
					buffer[0] = 1;
				} else {	
					buffer[0] = 0;
				}	
				buffer[1] = 0 - left_count + right_count;	
				buffer[2] = 0 - up_count + down_count;
				if (buffer[0] | buffer[1] | buffer[2])
					usb_send_data(1, (uns8 *)&buffer, 3, /*first*/ 0); // ep 1

			}
			left_count = right_count = up_count = down_count = center_count = 0;
		}	
	}	
}

void main() {

		
	configure_system();
	
	delay_ms(100);
	serial_print_str("\n\nEA PLT-1003 PIC USB Mouse demo\n");
	serial_print_str( "\n<");
	serial_print_str(__TIME__);
	serial_putc(' ');
	serial_print_str(__DATE__);
	serial_print_str(">\n");

	timer_start_0();	// kick that timer off...
	
	left_count = right_count = up_count = down_count = center_count = check_count = 0;
	serial_print_str("Attaching to USB...");
	
	usb_enable_module();
	
	while (usb_get_state() != st_CONFIGURED);
	
	serial_print_str("Done!\n");

	for(;;) {

		term_process();
		mouse_process();

	}
	// main
}