//-----------------------------------------------------
// Pic Pack library
// 
// mrf24j50_demo.c
//
//
// Ian Harris 2009
// imharris [at] gmail.com
//
// Released under the "do whatever you like with this
// but if it breaks, you get to keep both pieces" license.
//-----------------------------------------------------


// Pic Pack includes
#include "config.h"
#include "pic_utils.h"
#include "pic_serial.h"
#include "mrf24j40.h"
#include "spi_hw.h"

// System includes
#include "string.h"
#include "stdlib.h"
#include "config.h"

// Don't forget your config.h!

// Put received characters in here to see
// if they make a command
char serial_buffer[6]= '\0';

// See if we need to do a task
uns8 do_task = 0;

// Flag if we received something
bit receive = 0;

// Dummy variable to show we can set it 
uns8 send_to = 0;


bit promiscuous_mode = 0;

// Interrupt routine - - - - - - - - - -

void interrupt() {
	
	if (test_bit(intcon, INT0IF)) {
		clear_bit(intcon, INT0IF);
		mrf24j40_handle_int_isr();
	}	
	serial_handle_tx_isr();
	serial_handle_rx_isr();
}

// Process_rx
//
// Keep adding characters to our input buffer
// when we enter a command, then act on it

void process_rx() {
	
	uns8 len,	// length of string
		 rec;	// received character
	uns8 first_char;
	
	rec = serial_getc();	// get the character from the fifo
	
	serial_putc(rec);	// print it out so we can see what we typed
	
	if (rec == MAGIC_BOOSTBLOADER_REQUEST) {
		boostbloader();
	}

	if (rec == '\r') {	// did we press return?
		first_char = serial_buffer[0];
		switch (first_char) {
			case 'a':
				do_task = 1;
				break;
		    case 'x':
				do_task = 2;
				break;
			case 'c':
				do_task = 3;
				break;
			case 'p':
				do_task = 5;
				break;	
			case 'r':
				do_task = 6;
				break;								
			case 'd':
				do_task = 4;
				break;
			case 't':
				do_task = 7;
				break;
			default:
				// More complicated example
				// Let's set a variable based on input
				// You can type:
				// s05
				// And press enter, it will set "send_to" to this (in hex)
				if ((first_char == 's') && (strlen(serial_buffer)==3)) {
					send_to = atoui_hex(&serial_buffer[1]);
					serial_print_str(" Set send_to to ");
					serial_print_int(send_to);	// Of course, this could be anything
					serial_putc('\n');
				};
		}		
		serial_buffer[0] = '\0';	// clear the buffer
        serial_putc('\n');	// print new line
        serial_putc('>');	// print the prompt
	} else {
		len = strlen(serial_buffer);	// add the character to our line input
		serial_buffer[len] = rec;
		serial_buffer[len+1] = '\0';
	}

}

// configure_system
//
// Get the pic ready for everything we want to do

void configure_system() {
	    
	kill_interrupts();	// turn off interrupts just in case

	turn_analog_inputs_off();	// kill those pesky analogue inputs

	clear_bit(intcon, INT0IF);	// in case it was triggered already
	clear_bit(intcon2, INTEDG0);	// Need falling edge for mrf24j40 defaults
	set_bit(intcon, INT0IE);	// turn on rb0/int0 interrupts
	spi_hw_setup_io();
	spi_hw_init();
	mrf24j40_setup_io();
	mrf24j40_init();
		
	serial_setup(BRGH_HIGH_SPEED, SPBRG_19200);

	turn_peripheral_ints_on();
	turn_global_ints_on();
}


// Handle any command line tasks we need to do
// Very handy to print out variables, do commands,
// print out some debug, etc...

void handle_tasks() {
	
	uns8 data[10];
	uns8 EA_1[8] = { 0, 0, 0, 0, 0, 0, 0, 1 };
	uns8 EA_2[8] = { 0, 0, 0, 0, 0, 0, 0, 2 };
	
	switch (do_task) {
		case 1:
			serial_print_str("Xmiting to 1... ");
			data[0] = '!';
			data[1] = 'C';
			mrf24j40_transmit_to_extended_address(&EA_1, &data, 2);
			break;
		case 2:
			serial_print_str("Xmiting... ");
			data[0] = '$';
			data[1] = 'B';
			mrf24j40_transmit(&data, 2);
			
			break;
		case 3:
			serial_print_str("Setting address to 1\n");
			mrf24j40_set_extended_address(&EA_1);
			break;
		case 4:
			serial_print_str("Setting address to 2\n");
			mrf24j40_set_extended_address(&EA_2);
			break;
		case 5:
			serial_print_str("Setting pan id = 5\n");
			mrf24j40_set_pan_id(0x05);
			break;
		case 6:
			if (promiscuous_mode) {
				serial_print_str("Prom OFF\n");
				promiscuous_mode = 0;
				mrf24j40_short_addr_write(RXMCR, 0x00); 
			} else {
				promiscuous_mode = 1;
				serial_print_str("Prom ON\n");
				mrf24j40_short_addr_write(RXMCR, 0x01); // promiscuous mode, no coord
			}
			break;
			
		case 7:
			/*mrf24j40_long_addr_write(0x80, 0xff);
			uns8 val = mrf24j40_long_addr_read(0x80);
			serial_print_str("Read from 0x080 = ");
			serial_print_int_hex(val);
			serial_print_nl(); */
			uns8 lowest_channel = mrf24j40_scan_for_lowest_channel_ed();

			serial_print_str("Lowest channel = ");
			serial_print_int(lowest_channel);
			serial_print_nl();		
			break;
	} 	// switch		

	do_task = 0;
}

void main() {
	
	configure_system();
	delay_ms(100);
	serial_print_str("\n\nMRF24J40 test terminal\n");
	serial_print_str( "\n<");
	serial_print_str(__TIME__);
	serial_putc(' ');
	serial_print_str(__DATE__);
	serial_print_str(">\n");

	serial_print_str("Commands: a,x,c,n,s00\n");


	while(1){

		if (serial_rx_avail()) {
			process_rx();
		}
		handle_tasks();	  

	}	// while (1)

	
}	// main

