//-----------------------------------------------------
// Pic Pack library
// 
// somo_14d.c
//
// 4D Systems embedded SOund MOdule 1.
//
// Ian Harris 2009
// imharris [at] gmail.com
//
// Released under the "do whatever you like with this
// but if it breaks, you get to keep both pieces.
//-----------------------------------------------------


// Pic Pack includes
#include "config.h"
#include "pic_utils.h"
#include "pic_serial.h"
#include "somo_14d.h"

// System includes
#include "string.h"
#include "stdlib.h"
#include "config.h"

// Don't forget your config.h!

#define TASK_PLAY_PAUSE		1
#define TASK_STOP			2
#define TASK_SET_FILE_ID	3
#define TASK_SET_VOLUME		4

// Put received characters in here to see
// if they make a command
char serial_buffer[6]= '\0';

// See if we need to do a task
uns8 do_task = 0;

// Flag if we received something
bit receive = 0;

// Variables accessable from interrupt and main routine
uns8 volume = 7;
uns8 file_id = 0;


// Interrupt routine - - - - - - - - - -

void interrupt() {
	
		serial_handle_tx_isr();
		serial_handle_rx_isr();
}

// Process_rx
//
// Keep adding characters to our input buffer
// when we enter a command, then act on it

void process_rx() {
	
	uns8 len,	// length of string
		 rec;	// received character
	uns8 first_char;
	
	rec = serial_getc();	// get the character from the fifo
	
	serial_putc(rec);	// print it out so we can see what we typed

	if (rec == '\r') {	// did we press return?
		first_char = serial_buffer[0];
		switch (first_char) {
			case 'p':
				do_task = TASK_PLAY_PAUSE;
				break;
		    case 's':
				do_task = TASK_STOP;
				break;
			default:
				if ((first_char == 'f') && (strlen(serial_buffer)==3)) {
					file_id = atoui_dec(&serial_buffer[1]);
					serial_print_str(" Set file to ");
					serial_print_int(file_id);
					serial_putc('\n');
					do_task = TASK_SET_FILE_ID;
				} else if ((first_char == 'v') && (strlen(serial_buffer)==2)) {
					volume = serial_buffer[1] - '0';
					serial_print_str(" Set volume to ");
					serial_print_int(volume);
					serial_putc('\n');
					do_task = TASK_SET_VOLUME;
				}	
		}		
		serial_buffer[0] = '\0';	// clear the buffer
        serial_putc('\n');	// print new line
        serial_putc('>');	// print the prompt
	} else {
		len = strlen(serial_buffer);	// add the character to our line input
		serial_buffer[len] = rec;
		serial_buffer[len+1] = '\0';
	}

}

// configure_system
//
// Get the pic ready for everything we want to do

void configure_system() {
	    
	kill_interrupts();	// turn off interrupts just in case

	turn_analog_inputs_off();	// kill those pesky analogue inputs
	
	serial_setup(BRGH_HIGH_SPEED, SPBRG_9600);
	somo_14d_setup_io();
	turn_peripheral_ints_on();
	turn_global_ints_on();
}


// Handle any command line tasks we need to do
// Very handy to print out variables, do commands,
// print out some debug, etc...

void handle_tasks() {
	switch (do_task) {
		case TASK_PLAY_PAUSE:
			somo_14d_play_pause();
			break;
		case TASK_STOP:
			somo_14d_stop();
			break;
		case TASK_SET_FILE_ID:
			somo_14d_set_file_id(file_id);
			break;
		case TASK_SET_VOLUME:
			somo_14d_set_volume(volume);
			break;
	} // switch		

	do_task = 0;
}

void main() {
	
	configure_system();
	delay_ms(200);
	serial_print_str("\n\nSOMO 14D Demo\n");
	serial_print_str( "\n<");
	serial_print_str(__TIME__);
	serial_putc(' ');
	serial_print_str(__DATE__);
	serial_print_str(">\n");

	serial_print_str("Commands:\np Play/pause\ns Stop\nvX Set volume to X (0-7)\nfXX Set file to XX\n\n");

	for(;;) {	// endless loop

		if (serial_rx_avail()) {
			process_rx();
		}
		handle_tasks();	  

	}	// while (1)

	
}	// main

