//-----------------------------------------------------
// Pic Pack library
// 
// weather board test.c
//
//
// (c) Ian Harris 2010
// imharris [at] gmail.com
//
// 
//-----------------------------------------------------


// Pic Pack includes
#include "config.h"
#include "pic_utils.h"
#include "pic_serial.h"
#include "mrf24j40.h"
#include "spi_hw.h"
#include "wpan.h"
#include "pic_timer.h"
#include "pic_tick.h"
#include "convert.h"

#include "its_mode2.h"
#include "its_common.h"

// System includes
#include "string.h"
#include "stdlib.h"

#include "i2c.h"
#include "tmp75.h"
#include "ms5540.h"

#define TMP75_ADDR 0b00000000

bit temp_requested = 0;
uns16 request_id;

// Put received characters in here to see
// if they make a command
char serial_buffer[6]= '\0';

// See if we need to do a task
uns8 do_task = 0;

// Flag if we received something
bit receive = 0;

// Dummy variable to show we can set it 
uns8 send_to = 0;


bit promiscuous_mode = 0;

// Interrupt routine - - - - - - - - - -

void interrupt() {
	
	if (test_bit(intcon, INT0IF)) {
		clear_bit(intcon, INT0IF);
		mrf24j40_handle_isr();
	}	
	serial_handle_tx_isr();
	serial_handle_rx_isr();
	timer_handle_0_isr();	
}


	


// Process_rx
//
// Keep adding characters to our input buffer
// when we enter a command, then act on it

void process_rx() {
	
	uns8 len,	// length of string
		 rec;	// received character
	uns8 first_char;
	
	rec = serial_getc();	// get the character from the fifo
	
	serial_putc(rec);	// print it out so we can see what we typed
	
	if (rec == MAGIC_BOOSTBLOADER_REQUEST) {
		boostbloader();
	}

	if (rec == '\r') {	// did we press return?
		first_char = serial_buffer[0];
		switch (first_char) {
			case '1':
				do_task = 1;
				break;
		    case '2':
				do_task = 2;
				break;
		    case '3':
				do_task = 9;
				break;
			case 'c':
				do_task = 3;
				break;
			case 'd':
				do_task = 4;
				break;
			case 'e':
				do_task = 8;
				break;
			case 'p':
				do_task = 5;
				break;	
			case 'r':
				do_task = 6;
				break;								
			case 't':
				do_task = 7;
				break;
			case 'q':
				do_task = 10;
				break;
			case 'h':
				do_task = 11;
			case 'w':
				do_task = 12;
				break;
			case '=':
				do_task = 13;
				break;
			default:
				// More complicated example
				// Let's set a variable based on input
				// You can type:
				// s05
				// And press enter, it will set "send_to" to this (in hex)
				if ((first_char == 's') && (strlen(serial_buffer)==3)) {
					send_to = atoui_hex(&serial_buffer[1]);
					serial_print_str(" Set send_to to ");
					serial_print_int(send_to);	// Of course, this could be anything
					serial_putc('\n');
				};
		}		
		serial_buffer[0] = '\0';	// clear the buffer
        serial_putc('\n');	// print new line
        serial_putc('>');	// print the prompt
	} else {
		len = strlen(serial_buffer);	// add the character to our line input
		serial_buffer[len] = rec;
		serial_buffer[len+1] = '\0';
	}

}

// configure_system
//
// Get the pic ready for everything we want to do

void configure_system() {
	    
	kill_interrupts();	// turn off interrupts just in case

	turn_analog_inputs_off();	// kill those pesky analogue inputs

	clear_bit(intcon, INT0IF);	// in case it was triggered already
	clear_bit(intcon2, INTEDG0);	// Need falling edge for mrf24j40 defaults
	set_bit(intcon, INT0IE);	// turn on rb0/int0 interrupts
	
	spi_hw_setup_io();
	spi_hw_init();
	mrf24j40_setup_io();
	mrf24j40_init();
	mrf24j40_set_channel(15);
	#ifdef _PIC16
		timer_setup_0(TIMER_8BIT_MODE, TIMER_PRESCALER_1_TO_8, 0xff - 250 - 1);	// 1ms at 8Mhz
	#endif

	#ifdef _PIC18
		timer_setup_0(TIMER_16BIT_MODE, TIMER_PRESCALER_OFF, 0xffff - 5000);	// 1ms at 20Mhz
		//timer_setup_0(TIMER_16BIT_MODE, TIMER_PRESCALER_1_TO_2, 0xffff - 1500 + 7);	// 1ms at 12Mhz
		//timer_setup_0(TIMER_16BIT_MODE, TIMER_PRESCALER_1_TO_8, 0xffff - 375 + 1);	// 1ms at 12Mhz
	#endif

	tmp75_setup_io();
	// 12 bit temp, shutdown
	// OS R1 R0 F1 F0 POL TM SD
	// 0  1  1  0  0  0   0  0   
	tmp75_set_config(TMP75_ADDR, 0b01100000);
	
	serial_setup(SPBRG_115200);

	//adcon1 = 14;	// set an0 for analog input (pic18)
	
	adcon0 = 0b00001100;	// Select an3 as input to sample
	ansel  = 0b00001000;	// Choose an3 as analog input 
	anselh = 0x00;
	make_input(PORTA, 3);	// Make input

	set_bit(adcon2, ACQT2);
	set_bit(adcon2, ACQT1);
	set_bit(adcon2, ACQT0);
	
	
	set_bit(adcon2, ADCS2);	// fosc/64
	set_bit  (adcon2, ADCS1);
	clear_bit(adcon2, ADCS0);
	
	set_bit(adcon2, ADFM);	// Right justified
	adcon1 = 0;	// voltage refs are vdd and vss
	set_bit(adcon0, ADON);	// Turn on analog module
	
	// PR_MCLK = RB3	- E pwm 2 output (ccp2)
	// PR_SCLK = RB4
	// PR_DIN  = rb2
	// PR_DOUT = rb1
	
	// VOUT=(VSUPPLY)(0.00636(sensor RH) + 0.1515), typical at 25 C 	
	// True RH = (Sensor RH)/(1.0546 0.00216T), T in C

	// test out the pwm stuff
	
	// set tris bit
	make_input(PORTB, 3);
	
	// set pwm period by loading pr2 register
	pr2 = 121; // = 488 inc cleared dc2b bits = 32khz at 16Mhz
	
	//Configure the CCP module for the PWM mode
	//by loading the CCPxCON register with the
	//appropriate values. 
	
	 
	// 11xx of CCP2M<3:0> sets pwm mode
	set_bit(ccp2con, CCP2M3);	// pwm mode
	set_bit(ccp2con, CCP2M2); 
		
	//Set the PWM duty cycle by loading the CCPRxL
	//register and CCPx bits of the CCPxCON register.
	ccpr2l = 61; // = 50% of 488 inc dc2bx bits = 32khz at 16mhz clock

	//ccp2con 
	//dc2b1 and dc2b0 are the two lsbs of the 10 bit duty cycle
	clear_bit(ccp2con, DC2B1);
	clear_bit(ccp2con, DC2B0);


	//Configure and start Timer2:
	// Clear the TMR2IF interrupt flag bit of the PIR1 register.
	clear_bit(pir1, TMR2IF);
	
	// Set the Timer2 prescale value by loading the T2CKPS bits of the T2CON register.
	clear_bit(t2con, T2CKPS1);	// 01 = prescale = 4 (00 prescale = 0, 1x prescale = 16)
	set_bit(t2con, T2CKPS0);
	
	// Enable Timer2 by setting the TMR2ON bit of the T2CON register.
	set_bit(t2con, TMR2ON);

	//Enable PWM output after a new PWM cycle has started:
	// Wait until Timer2 overflows (TMR2IF bit of the PIR1 register is set).
	// Enable the CCPx pin output driver by clearing the associated TRIS bit.

	// bugger it
	make_output(PORTB, 3);


	// duty cycle
	// ccpr2l<7:0> -  dc2b<1:0>
	//The 8-bit timer TMR2 register is concatenated
	//with the 2-bit internal system clock (FOSC), or
	//2 bits of the prescaler, to create the 10-bit time
	//base.
	
//	CCPRxL:DCxB<1:0> = half of PR2

	
	turn_peripheral_ints_on();
	turn_global_ints_on();

	ms5540_setup_io();

}


// Handle any command line tasks we need to do
// Very handy to print out variables, do commands,
// print out some debug, etc...

void handle_tasks() {
	
	uns8 data[10];
	uns8 EA_1[8] = { 0, 0, 0, 0, 0, 0, 0, 1 };
	uns8 EA_2[8] = { 0, 0, 0, 0, 0, 0, 0, 2 };
	uns8 EA_3[8] = { 0, 0, 0, 0, 0, 0, 0, 3 };
	
	switch (do_task) {
		case 1:
			serial_print_str("TX1... ");
			data[0] = 'T';
			data[1] = 'O';
			// we just happen to know the device is 0x2020
			its2_router_queue_packet(0x0001, ITS_GENERIC_DATA, &data, 2, 1);

			break;
		case 2:
			serial_print_str("TX2.... ");
			data[0] = 'o';
			data[1] = 't';
			its2_router_queue_packet(0x0002, ITS_GENERIC_DATA, &data, 2, 1);
			
			break;
		case 9:
			serial_print_str("TX3.... ");
			data[0] = 'T';
			data[1] = 't';
			its2_router_queue_packet(0x0003, ITS_GENERIC_DATA, &data, 2, 1);
			
			break;
		case 3:
			serial_print_str("Setting address to 1\n");
			mrf24j40_set_extended_address(&EA_1);
			its2_router_init(0x0001, ITS_NETWORK_ID);
			
			break;
		case 4:
			serial_print_str("Setting address to 2\n");
			mrf24j40_set_extended_address(&EA_2);
			its2_router_init(0x0002, ITS_NETWORK_ID);
			break;
		case 8:
			serial_print_str("Setting address to 3\n");
			mrf24j40_set_extended_address(&EA_3);
			its2_router_init(0x0003, ITS_NETWORK_ID);
			break;
		case 5:
			serial_print_str("Setting pan id = 5\n");
			mrf24j40_set_pan_id(0x05);
			break;
		case 6:
			if (promiscuous_mode) {
				serial_print_str("Prom OFF\n");
				promiscuous_mode = 0;
				mrf24j40_short_addr_write(RXMCR, 0x00); 
			} else {
				promiscuous_mode = 1;
				serial_print_str("Prom ON\n");
				mrf24j40_short_addr_write(RXMCR, 0x01); // promiscuous mode, no coord
			}
			break;
		case 10:
				serial_print_str("debug mode\n");
				debug_module = 1;
				uns8 config = tmp75_get_config(TMP75_ADDR);
				serial_print_str("config = ");
				serial_print_int_hex(config);
				serial_print_nl();
				uns16 temp = tmp75_get_temp(TMP75_ADDR);
				serial_print_str("Temp = ");
				serial_print_int_hex_16bit(temp);
				serial_print_nl();
				uns8 int_part = temp >> 8;
				uns8 fract_part = temp & 0x00ff;
				char buffer[10];
				uns8 dec_value = convert_to_dec2(fract_part>>4);
				serial_print_str(temp_to_str(int_part, dec_value, buffer));
				serial_print_nl();
				break;
		case 11:
				its_print_devices();
				break;
			
		case 7:
			its2_print_queue();
			/*mrf24j40_long_addr_write(0x80, 0xff);
			uns8 val = mrf24j40_long_addr_read(0x80);
			serial_print_str("Read from 0x080 = ");
			serial_print_int_hex(val);
			serial_print_nl(); */
			/*uns8 lowest_channel = mrf24j40_scan_for_lowest_channel_ed();

			serial_print_str("Lowest channel = ");
			serial_print_int(lowest_channel);
			serial_print_nl();	*/	
			break;
		case 12:
				/*serial_print_str("reading adc 1.2v ...");
				adcon0 = 0b00111101;	// Select 1.2v fixed voltage as input to sample
				delay_us(20); 	// wait acquisition time
				set_bit(adcon0, DONE);
				while (test_bit(adcon0, DONE)) {
				}
				uns16 analog_1v2 = adresh;
				analog_1v2 = analog_1v2 << 8;
				analoganalog_1v2_input = analog_1v2 + adresl;
				serial_print_int(analog_1v2);
				serial_print_nl();
				serial_print_str("Reading hih-5030: ");
				adcon0 = 0b00001101;	// Select an3 fixed voltage as input to sample
				delay_us(20); 	// wait acquisition time
				*/
				set_bit(adcon0, DONE);
				while (test_bit(adcon0, DONE)) {
				}
				uns16 analog_input = adresh;
				analog_input = analog_input << 8;
				analog_input = analog_input + adresl;
				serial_print_int(analog_input);
				serial_print_nl();
				
				// so if 1.2v = analog_1v2 
				// 12v = 10 * analog1v2
				// 1200v  = 100 * 10 * analog1v2
				// 1000000v  = 100000 * analog1v2 / 12
				
				// so analog_input
				//=((D9/3.3) - 0.1515)/0.00636
				
				
				
				
			break;
		case 13:
				serial_print_str("get temp/pr...\n");
				ms5540_calc_temp_and_pressure();
				
				
				
				
			break;	
				

	} 	// switch		

	do_task = 0;
}

void its2_router_receive_callback(uns16 device_id, uns8 *data, uns8 data_length) {
		serial_print_str("Controller receives from ");
		serial_print_int(device_id);
		serial_print_nl();
		request_id = device_id;
		temp_requested = 1;
		//ds1631_convert_temp(DS1631_ADDR);	
}

void main() {

	uns8 my_ea[8] = { 0, 0, 0, 0, 0, 0, 0, 2};
	
	configure_system();
	
	delay_ms(100);
	delay_ms(100);
	
	serial_print_str("\n\nWeather board test\n");
	serial_print_str( "\n<");
	serial_print_str(__TIME__);
	serial_putc(' ');
	serial_print_str(__DATE__);
	serial_print_str(">\n");

	serial_print_str("Commands: a,x,c,n,s00\n");

//	its2_router_init(/* device */ 0x1010, /* network id */ 0x5000);
	serial_print_str("Setting address to 3\n");
	mrf24j40_set_extended_address(&my_ea);
	its2_router_init(0x0002, ITS_NETWORK_ID);
	
	timer_start_0();
	
	ms5540_init();
	
	uns16 temp;
	
	for(;;) {

		if (serial_rx_avail()) {
			process_rx();
		}

		wpan_process();	
		
		its2_router_process();
		
		handle_tasks();	  

		if (temp_requested) {	// temp requested, so lets see if it has finished
			uns8 config = 0; //ds1631_get_config(DS1631_ADDR);
			//if (test_bit(config, 7)) {	// the DONE bit
				temp_requested = 0;
				temp = tmp75_get_temp(TMP75_ADDR);
				serial_print_str("Temp = ");
				serial_print_int_hex_16bit(temp);
				serial_print_nl();
				its2_router_queue_packet(request_id, ITS_GENERIC_DATA, (uns8*)&temp, 2, ITS2_FLAG_ACK);

			//}	
		}


	}	// while (1)

	
}	// main

