/*

Copyright (c) 2010, Embedded Adventures, www.embeddedadventures.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

- Neither the name of Embedded Adventures nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
THE POSSIBILITY OF SUCH DAMAGE.

Contact us at admin@embeddedadventures.com

*/


/**
	\file hmc6352.h
	\brief Routines for communicating with the hmc6352 digital compass
*/

#ifndef __hmc6352_h
#define __hmc6352_h

#include "pic_utils.h"

#define hmc6352_device_addr 0x42
#define hmc6352_read		0x01
#define hmc6352_write		0x00

#define hmc6352_write_to_eeprom	0x77	// 'w'
#define hmc6352_read_from_eeprom	0x72	// 'r'
#define hmc6352_write_to_ram	0x47	// 'G'
#define hmc6352_read_from_ram	0x67	// 'g'
#define hmc6352_sleep_cmd		0x53	// 'S'
#define hmc6352_wake_cmd		0x57	// 'W'
#define hmc6352_update_bridge_cmd	0x4F	// 'O'
#define hmc6352_enter_cal_cmd		0x43	//	'C'
#define hmc6352_exit_cal_cmd		0x45	// 'E'
#define hmc6352_save_op_mode_cmd	0x4c	// 'L'
#define hmc6352_get_data_cmd		0x41	// 'A'

#define hmc6352_ee_slave_addr	0x00	// default 0x42
#define hmc6352_ee_x_offset_msb	0x01
#define hmc6352_ee_x_offset_lsb	0x02	
#define hmc6352_ee_y_offset_msb	0x03
#define hmc6352_ee_y_offset_lsb	0x04
#define hmc6352_ee_time_delay	0x05	// default 0x01 (0-255ms)
#define hmc6352_num_summed		0x06	// default 0x04 (1-16)
#define hmc6352_software_ver	0x07	// > 0x01
#define hmc6352_op_mode			0x08	// default 0x50

#define hmc6352_mode_standby	0x00
#define hmc6352_mode_query		0x01
#define hmc6352_mode_continuous	0x02

#define hmc6352_ram_op_mode_control		0x74
#define hmc6352_ram_output_data_control	0x4e


// Op mode syntax
//
// Continuous Mode Measurement rate
//
//	d6	d5
//	0	0	1  Hz
//	0	1	5  Hz
//	1	0	10 Hz
//	1	1	20 Hz


// Periodic set/reset
//
//	d4
//	0	Off
//	1	On

// Operational Mode Value
//
//	d1	d0
//	0	0	Standby mode
//	0	1	Query	mode
//	1	0	Continuous mode
//	1	1	Not allowed


// Output data control

//	d2	d1	d0
//	0 	0	0	Heading Mode (degrees in 0 to 3599 over two bytes)
//	0 	0	1	Raw Magnetometer X Mode (10bit 2s complement sensor)
//	0	1	0	Raw Magnetometer Y Mode
//	0	1	1	Magnetometer X Mode (sensor with offset and scaling applied)
//	1	0	0	Magnetometer Y Mode



// on power up - heading output mode
// 

void hmc6352_write_eeprom(uns8 addr, uns8 data);
uns8 hmc6352_read_eeprom(uns8 addr);
void hmc6352_write_ram(uns8 addr, uns8 data);
uns8 hmc6352_read_ram(uns8 addr);

void hmc6352_sleep();
void hmc6352_wake();
void hmc6352_update_bridge_offsets();
void hmc6352_enter_cal();
void hmc6352_exit_cal();
void hmc6352_save_op_mode();
uns16 hmc6352_get_data();

void hmc6352_set_mode(uns8 mode);
void hmc6352_setup_io();




#endif
