/*

Copyright (c) 2010, Embedded Adventures, www.embeddedadventures.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

- Neither the name of Embedded Adventures nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
THE POSSIBILITY OF SUCH DAMAGE.

Contact us at admin@embeddedadventures.com

*/


//#define DEBUG_MODULE
/* to do:

- address grandfathering
- callbacks
- //!! notes
- route failure messages
- what happens when our routed packet fails? Can we kill all routes
  and try again?

*/

#include "its_mode2.h"
#include "its_common.h"
#include "wpan.h"
#include "mrf24j40.h"

#include "config.h"
#include "pic_serial.h"
#include "pic_tick.h"
#include "pic_timer.h"
#include "memory.h"

#include "debug.h"

its2_state state = STATE_STARTUP;

bit debug_module =0;
bit queue_processing = 0;

uns8 controller_handle;
uns16 tick_marker;
uns16 state_timeout;

uns8 channel;
bit its2_transmitting = 0;
uns8 its2_seen_index;

static seen_packet    its2_seen_list[ITS2_SEEN_LIST_SIZE];
static queued_item    its2_tx_queue[ITS2_TX_QUEUE_SIZE];

// ---------------------------------------------------------------------------
// Mode 2 common routines 
// ---------------------------------------------------------------------------

void turn_off_mrf_interrupts() {
	//!!clear_bit(intcon, INT0IE);
	nop();
	nop();
	nop();
	nop();
}

void turn_on_mrf_interrupts() {
	//!!set_bit(intcon, INT0IE);
}	


void its2_setup_io() {
	wpan_setup_io();
}

uns8 its2_find_free_queue_slot() {

bit found;
uns8 count;

	found = 0;
	for (count = 0; count < ITS2_TX_QUEUE_SIZE; count++) {
		if (its2_tx_queue[count].flag  == ITS2_FLAG_DELETED) {
			found = 1;
			break;
		}
	}
	if (found) {
		return count;
	} else {
		return 0xff;
	}
}		

its2_result its2_router_handle_association(uns16 pan_id, uns16 its_device_id) {
	// see if this device is in our list, if not, add it
its_device_handle device_handle;

	device_handle = its_get_device_handle(its_device_id);
	
	if (device_handle == ITS_DEVICE_NONE) {
		// in mode 1 we assume that 
		device_handle = its_add_local_device(/* device */ its_device_id, pan_id, /* short addr */ its_device_id);
		if (device_handle == ITS_DEVICE_NONE) {
			// panic!
			debug_str("Too many devices! association failed");
			return RESULT_FAILED;
		} else {
			// send association response
			its_transmit_to_handle(device_handle, ITS_ASSOC_RES, /* nil data */ 0, /* zero length */ 0);
			return RESULT_SUCCESSFUL;
			// although strictly speaking we should wait for the ack...
		}
			
	}
}	



#ifdef ITS_MODE2_ROUTER

// router version

							
							
uns8 its2_update_route(queued_item *item) {

its_device_info *device_info;
its_device_info *router_info;
uns16 route_list[ITS2_MAX_HOP_COUNT];
uns8	router_handle;
uns8 count;
uns8 count_hops;
uns16 hop;

	debug_str("\n Routing: ");
	debug_int_hex_16bit(item->dest_its_device_id);
	hop = item->dest_its_device_id; // was device_id;
	item->dest_device_handle = its_get_device_handle(hop);
	if (item->dest_device_handle != ITS_DEVICE_NONE) {	// known... now find the route
		debug_str(" known ");
		count_hops = 0;
		while (count_hops < ITS2_MAX_HOP_COUNT) {
			router_handle = its_get_device_handle(hop);
			router_info = its_get_device_info(router_handle);
			if (router_info->addr.remote.remote_indicator != 0xffff) { // local?
				item->dest_device_handle = router_handle;
				debug_int_hex_16bit(hop);
				debug_str(" (local) ");
				break;
			} else {	// remote
				debug_str(" via ");
				debug_int_hex_16bit(router_info->addr.remote.prior_device_id);
				route_list[count_hops] = router_info->addr.remote.prior_device_id;
				hop = router_info->addr.remote.prior_device_id;
				count_hops++;	
			}
		}
		if (count_hops == ITS2_MAX_HOP_COUNT) {
			debug_str(" MAX HOPS EXCEEDED! ");
			return ITS2_UPDATE_ROUTE_FAIL;
		}	
		
		// okay now we have a route...
		item->packet.num_routes = count_hops;
		count = 0;
		debug_str(" Route via [");
		// reverse order of route
		while (count_hops > 0) {
			debug_var(" ", route_list[count_hops-1]);
			item->packet.routers[count] = route_list[count_hops-1];
			count++;
			count_hops--;
		}
		debug_str("] ");
		item->status = QS_READY_TO_SEND;
	} else { // unknown!
//		if ((device_id == 0xffff) || (item->dest_its_device_id == 0xffff) ) { // broadcast
		if (item->dest_its_device_id == 0xffff) { // broadcast
				item->packet.num_routes = 0;
			item->status = QS_READY_TO_SEND;
		} else {
			debug_str("Unknown, trying local ");
			item->status = QS_WAITING_ON_LOCAL_ADDR;
		}	
	}
	return ITS2_UPDATE_ROUTE_SUCCESS;
}

void its2_respond_ack(uns16 its_device_id, uns8 sequence) {
	debug_str("Sending ACK");
	its2_router_queue_packet(its_device_id, ITS_ACK, &sequence, 1, ITS2_FLAG_NO_ACK);
}

uns8 its2_seen_packet(uns16 its_source_id, uns8 sequence) {

	for (uns8 count=0; count < ITS2_SEEN_LIST_SIZE; count++) {
		if ((its2_seen_list[count].its_source_id == its_source_id) &&
		    (its2_seen_list[count].sequence == sequence)) {
			return 1;
		}
	}
	return 0;
}

its2_add_to_seen_list(uns16 its_source_id, uns8 sequence) {
	
	its2_seen_list[its2_seen_index].its_source_id = its_source_id;
	its2_seen_list[its2_seen_index].sequence = sequence;
	its2_seen_index++;
	if (its2_seen_index == ITS2_SEEN_LIST_SIZE) {
		its2_seen_index = 0;
	}
}		



void wpan_data_received_callback(wpan_address *addr, uns8 *data, uns8 data_size, uns8 lqi, uns8 rssi) {

//uns8 pkt_type;
uns8 device_handle;
uns8 count;
//uns8 hop_count;
its2_packet pkt;
uns16 source_pan_id;
uns16 source_sa;

	turn_off_mrf_interrupts();

	#ifdef debug_on
		wpan_print_address(addr);
	#endif
	
	debug_int(data_size);
	debug_str(" bytes (");
	
//	debug_int(data_size);
//	debug_str(" bytes (");
	#ifdef debug_on
	
/*		for (count = 0; count < data_size + 4; count++) {
			debug_int_hex(data[count]);
			debug_putc(' ');
			if ((data[count] > 31) && (data[count] < 127)) {
				debug_putc(data[count]);
			} else {
				debug_putc('?');
			}	
			debug_putc(' ');
		}
!!
*/
		debug_var(") lqi=", lqi);
		debug_var(" rssi=", rssi);
	
	#endif
	
	if (debug_module) {
		if (addr->source_sa == 0x0001) {
			debug_str(" IGNORING ");
			return;
		}	
	}

	// see if it is an ITS packet
	if ((data_size > 2) && (data[POS_KEY1] == 'I') && (data[POS_KEY2] == 'T')) {
		debug_str("ITS Pkt!\n");
		uns8 length_header = data[POS_LENGTH_HEADER];
		uns8 length_data   = data[3+length_header+1];
		uns8 data_start    = 3+length_header; //+2;
		uns16 my_id = its_get_device_id();
		source_pan_id = addr->source_pan_id;
		source_sa = addr->source_sa;
		memcpy(/*dst*/  (void *)&pkt, // copy it in
				/*src*/  (void *)&data[POS_PKT_TYPE],	// &??
				/*len*/  data_size - POS_PKT_TYPE);
		its2_print_packet(&pkt);
		
		// find out if we are in the routing list
		
		bit i_am_in_route_list = 0;
		for (uns8 count = 0; count < pkt.hop_count; count++) {
			if (pkt.routers[count] == my_id) {
				i_am_in_route_list = 1;
			}
		}

		// regardless, let's keep track of who this is from
		// so long as we didn't send it, and we haven't previously routed it
		
		if ((pkt.its_source_id != my_id) && (!i_am_in_route_list)) { // not from me
		// !! what if it the last hop??
			if (pkt.hop_count == 0) { // it came direct
				debug_str("\n Device ");
				debug_int_hex_16bit(pkt.its_source_id);
				device_handle = its_get_device_handle(pkt.its_source_id);
		
				if (device_handle == ITS_DEVICE_NONE) {	// Don't know it
					// its packets are always short addresses
					debug_str(" (adding)");
					device_handle = its_add_local_device(/* device */ pkt.its_source_id, source_pan_id, source_sa);
					if (device_handle == ITS_DEVICE_NONE) {
						// panic!
						debug_str(" but table full\n");
					}
				} else { // it's already in the table
					debug_str(" known device\n");
				}
			} else {	// it's routed packet so unpick the routing list
				uns16 last_route = pkt.routers[pkt.hop_count-1];
				debug_var("Last route: ", last_route);
				device_handle = its_get_device_handle(last_route);
				if (device_handle == ITS_DEVICE_NONE) {	// Don't know it
					debug_str(" adding ");
					device_handle = its_add_local_device(/* device */ last_route, source_pan_id, source_sa);
				}

				// now run through routes if it was more than one hop
				if (pkt.hop_count > 1) {
					for (uns8 count = pkt.hop_count - 2; count != 255; count--) {
						device_handle = its_get_device_handle(last_route);
						if (device_handle == ITS_DEVICE_NONE) {	// Don't know it
							its_add_net_device(/* device */ last_route, /* previous */ pkt.routers[count]);
						}
						last_route = pkt.routers[count];
					}	
				}

				// okay, now we have to add the actual originator
				device_handle = its_get_device_handle(pkt.its_source_id);
				if (device_handle == ITS_DEVICE_NONE) {	// Don't know it
					debug_str(" (add rte)");
					device_handle = its_add_net_device(/* device */ pkt.its_source_id,/* previous */ last_route);
				}
				
			}
		} else {
			debug_str("NOTADDING");
		}	
		//!! how do we handle dead routes?
		debug_nl();	
		// Now decide what to do with it
		if (pkt.packet_type == ITS_ASSOC_REQ) {
			debug_str("ITS Assoc req\n");
			// normally we would check if we are in config mode; 
			// for the moment, let's do this all the time
			//uns16 network_id_req = pkt.its_network_id;
			//data[POS_NETWORK_H];	//msb of network from header
			//network_id_req <<= 8;
			//network_id_req += data[POS_NETWORK_L];	//lsb of network from header
			
			if (pkt.its_network_id == my_id) {	// it's my network
				// it's for me!
				//uns16 device_id_req = data[POS_SOURCE_H]; 	// msb source
				//device_id_req <<= 8;
				//device_id_req += data[POS_SOURCE_L];	// lsb source
				its2_router_handle_association( /* pan id */pkt.its_network_id, pkt.its_source_id);
			} else {
				debug_str("Not for us, for network: ");
				debug_int_hex_16bit(pkt.its_network_id);
			}	
		}	// ASSOC REQ
		else if (pkt.packet_type == ITS_LOCAL_DISCOVER_REQ) {
			debug_str(" Local disc");
			if (pkt.its_dest_id == my_id) {
				debug_str(" for me!");
				its2_respond_local_addr(pkt.its_source_id);
			} else {
				debug_str(" not for me");
			}	
		}  // LOCAL DISCOVER REQ 
		else if (pkt.packet_type == ITS_NET_DISCOVER_REQ) {	// NET_DISCOVER_REQ
			debug_str(" Net disc");
			if (pkt.its_dest_id == my_id) {
				debug_str(" for me!");
				its2_respond_net_addr(pkt.its_source_id);
			} else {
				debug_str(" not for me");
				
				// So rebroadcast if we haven't seen it before
				// Did it come from me in the first place?
				// or am I in the router list?
				
				// !! check hop count vs max hop count
				if ((pkt.its_source_id != my_id) &&	// not from me
				    (!i_am_in_route_list) &&			// not previously routed
				    (!its2_seen_packet(pkt.its_source_id, pkt.sequence))) {	// not seen before
						its2_rebroadcast_net_discover_req(&pkt);
				} else {
					debug_str(" NOT ");
				}	
				debug_str(" rebroadcasting ");		
			}	
		}  // LOCAL DISCOVER REQ
		else if (pkt.packet_type == ITS_LOCAL_DISCOVER_RES) {
			debug_str(" Local disc resp");
			// Got a response, now need to figure out who it is from
			for (uns8 count = 0; count < ITS2_TX_QUEUE_SIZE; count++) {
				if ((its2_tx_queue[count].flag  != ITS2_FLAG_DELETED) &&
			        (its2_tx_queue[count].packet.its_dest_id == pkt.its_source_id)) {
			        	debug_str(" Found match, changing status\n");
			        	its2_tx_queue[count].dest_device_handle = device_handle;
			        	its2_tx_queue[count].status = QS_READY_TO_SEND;
			        	break;
			    }
			}  	
				
		} // LOCAL DISCOVER RESPONSE
		else if (pkt.packet_type == ITS_GENERIC_DATA) {
			debug_str(" Generic data pkt");
			if (pkt.its_dest_id == my_id) { // it's for us!
				debug_str(" for us");
				// !! Need to send an ack
				its2_respond_ack(pkt.its_source_id, pkt.sequence);
				its2_router_receive_callback(pkt.its_source_id, &data[data_start], data_size - data_start);
			} else {
				debug_var(" route via ", pkt.routers[pkt.hop_count-1]);
				its2_forward_routed_packet(&pkt, &data[data_start], data_size - data_start);
			}
		}		// generic data
		else if (pkt.packet_type == ITS_ACK) {
			debug_str("Got ACK! seq=");
			debug_int(data[data_start]);
			debug_spc();
			if (pkt.its_dest_id == my_id) {
				bit found = 0;
				for (uns8 count = 0; count < ITS2_TX_QUEUE_SIZE; count++) {
					if ((its2_tx_queue[count].flag  != ITS2_FLAG_DELETED) &&
						(its2_tx_queue[count].packet.its_source_id == my_id) &&
						(its2_tx_queue[count].packet.sequence == data[data_start])) {
							debug_str(" Match! ");
							its2_tx_queue[count].status = QS_ACK_RECEIVED;
							found = 1;
							break;
					}
				}
				if (!found) {
					debug_str(" <Didn't find. Odd.>");
				}
			} else {
				debug_str(" Not for me, routing ");
				its2_forward_routed_packet(&pkt, &data[data_start], data_size - data_start);
			}	
		}	
		else if (pkt.packet_type == ITS_NET_DISCOVER_RES) {
			debug_str(" Net disc RES");
			if (pkt.its_dest_id == my_id) { // it's for us!
				debug_str(" for us!");
				bit found = 0;
				for (uns8 count = 0; count < ITS2_TX_QUEUE_SIZE; count++) {
					if ((its2_tx_queue[count].flag  != ITS2_FLAG_DELETED) &&
						(its2_tx_queue[count].packet.its_dest_id == pkt.its_source_id)) {
							debug_str(" Found match, updating route\n");
							uns8 update_route_result = its2_update_route(&its2_tx_queue[count]);
							if (update_route_result == ITS2_UPDATE_ROUTE_FAIL) {
								debug_str(" Route failed\n");
								its2_tx_queue[count].status = QS_ROUTING_FAILED;
							}	
							found = 1;
							break;
					}
				}
				if (!found) {
					debug_str(" <Didn't find. Odd.>");
				}
			} else {
				// we need to forward packet
				debug_var(" route via ", pkt.routers[pkt.hop_count-1]);
				its2_print_packet(&pkt);
				its2_forward_routed_packet(&pkt, &data[data_start], data_size - data_start);
			}
		}
		else {
				debug_var(" Pkt unknown type ", pkt.packet_type);
		}	// type of pkt		
	} // ITS packet
	
	turn_on_mrf_interrupts();
		
} // wpan_frame_received_callback

#else

// device version
void wpan_data_received_callback(wpan_address *addr, uns8 *data, uns8 data_size) {

uns8 count;
	wpan_print_address(addr);
	//wpan_print_frame_type(frame_type);
	debug_str(" Data: ");
	for (count = 0; count < data_size; count++) {
		debug_int_hex(data[count]);
		debug_putc(' ');
		if ((data[count] > 31) && (data[count] < 127)) {
			debug_putc(data[count]);
		} else {
			debug_putc('?');
		}	
		debug_putc(' ');
	}
	debug_nl();

	// check frame type
	if ((data_size > 2) && (data[0] == 'I') && (data[1] == 'T')) {
		debug_str("ITS Packet received\n");
		uns8 length_header = data[2];
		uns8 length_data   = data[3+length_header+1];
		uns8 data_start    = 3+length_header+2;
		// okay, but is it an its association request?
		if (data[3] == ITS_ASSOC_RES) {
			debug_str("Received association response\n");
			if (state == STATE_SEARCHING) {
				debug_str("Changing state to associated\n");
				uns16 controller_device_id = data[8]; // device id of controller
				controller_device_id <<= 8;
				controller_device_id += data[7];
				
				uns8 device_handle = its_get_device(controller_device_id);

				if (device_handle == ITS_DEVICE_NONE) {

					device_handle = its_add_device(/* device */ controller_device_id, addr->source_pan_id, controller_device_id);
					if (device_handle == ITS_DEVICE_NONE) {
						// panic!
						debug_str("Too many devices! association failed");
						state = STATE_UNASSOCIATED;
					} else {
						controller_handle = device_handle;
						state = STATE_ASSOCIATED;	// stop searching
					}
				} else { // it's already in the table
					state = STATE_ASSOCIATED;
				}	
					
			}	
		}	// association response
		else if (data[3] == ITS_GENERIC_DATA) {
			// is it for us?
			// who cares right now!
			// todo...
			debug_str("Generic data pkt\n");
			// just issue the callback
			uns16 device_id = data[8];	// msb
			device_id  <<= 8;
			device_id += data[7];	// lsb
			its2_device_receive_callback(&data[data_start], data_size - data_start);

		}	// GENERIC DATA		
	}
}
#endif

// ---------------------------------------------------------------------------
// Controller specific routines 
// ---------------------------------------------------------------------------

void its2_print_packet(its2_packet *pkt) {
	
	debug_str("(pkt type=");
	debug_int(pkt->packet_type);
	debug_str(" seq=");
	debug_int(pkt->sequence);
	debug_str(" net=");
	debug_int_hex_16bit(pkt->its_network_id);
	debug_str(" src=");
	debug_int_hex_16bit(pkt->its_source_id);
	debug_str(" dst=");
	debug_int_hex_16bit(pkt->its_dest_id);
	debug_str(" max_hop=");
	debug_int(pkt->max_hop_count);
	debug_str(" num_rts=");
	debug_int(pkt->num_routes);
	debug_str(" hop_cnt=");
	debug_int(pkt->hop_count);
	
	uns8 count;
	debug_str(" Rts=");
	for (count = 0; count < pkt->num_routes; count++) {
		if (count > ITS2_MAX_HOP_COUNT) {
			debug_str(" Too many routes! ");
			break;
		} else {
			debug_putc(' ');
			debug_int_hex_16bit(pkt->routers[count]);
		}	
	}	
	debug_str(") ");
}

its2_result its2_rebroadcast_net_discover_req(its2_packet *pkt) {

uns8 queue_slot;
queued_item *item;
	
	if (pkt->hop_count == pkt->max_hop_count) {
		debug_str("<Can't re-b: max hop count exceeded!>");
		return ROUTING_TOO_MANY_HOPS;
	}
	queue_slot = its2_find_free_queue_slot();
	if (queue_slot != ITS2_NO_AVAILABLE_SLOTS) {
		
		debug_var("<Qed>", queue_slot);
		item = &its2_tx_queue[queue_slot];
		
		item->flag = ITS2_FLAG_NO_ACK;	// it's ours!
		memcpy(/*dst*/  (void *)&item->packet, // copy it in
				/*src*/  (void *)pkt,
				/*len*/  11 + (pkt->num_routes * 2));
		item->dest_its_device_id = 0xffff;
		item->packet.routers[item->packet.hop_count] = its_get_device_id();
		item->packet.hop_count++;
		item->packet.num_routes++;
		item->sent_count = 0;
		item->dest_device_handle = ITS_DEVICE_NONE; // it's a broadcast
		item->data_length = 0;
		item->data = NULL;


		item->status = QS_READY_TO_SEND;
		
		its2_add_to_seen_list(item->packet.its_source_id, item->packet.sequence);

		return ITEM_QUEUED;
	} else {
		return QUEUE_FULL;
	}	

}	

its2_result its2_forward_routed_packet(its2_packet *pkt, uns8 *data, uns8 data_length) {

uns8 queue_slot;
queued_item *item;
void	*p;

	queue_slot = its2_find_free_queue_slot();
	
	if (queue_slot != ITS2_NO_AVAILABLE_SLOTS) {
	
		debug_var("<Qed>", queue_slot);
		debug_str(" data lengh=");
		debug_int(data_length);
		debug_spc();
	
		item = &its2_tx_queue[queue_slot];
		item->flag = ITS2_FLAG_NO_ACK;	// it's ours!
		memcpy(/*dst*/  (void *)&item->packet, // copy it in
				/*src*/  (void *)pkt,
				/*len*/  11 + (pkt->num_routes * 2));
	
		its2_print_packet(&item->packet);

		item->packet.hop_count++;

		// If hop_count now == num_routes then we are done routing
		// and have to go final (local) destination.
		// Otherwise, forward to next hop.
		
		if (item->packet.hop_count == item->packet.num_routes) {
			// Final desintation
			debug_str(" going to final=");
			item->dest_its_device_id = item->packet.its_dest_id;
		} else {
			// Forward to next hop
			debug_str(" going to next hop=");
			item->dest_its_device_id = item->packet.routers[item->packet.hop_count];
		}
			
		debug_int_hex_16bit(item->dest_its_device_id);

		item->sent_count = 0;
		item->dest_device_handle = its_get_device_handle(item->dest_its_device_id);
		
		// Do we know about it already?
		if (item->dest_device_handle == ITS_DEVICE_NONE) {
			// Not in our routing table. Let's see if it's local
			debug_str(" unknown. Looking for local. ");
			item->status = QS_WAITING_ON_LOCAL_ADDR;
		} else {
			// Check that it is really local
			its_device_info *device_info = its_get_device_info(item->dest_device_handle);
			
			if (device_info->addr.remote.remote_indicator == 0xffff) {
				debug_str(" Not local! Abandon ship ");
				item->flag = ITS2_FLAG_DELETED;	// delete it
				return NEXT_HOP_NOT_LOCAL;	// let our caller know
				// !! we should send something back to originator
				// to say that we couldn't route it
			} else {	
				debug_str(" found locally. ");
				item->status = QS_READY_TO_SEND;
			}	
		}	
		item->data_length = 0;
		item->data = NULL;
		
		// copy data...
		
		if (data_length > 0) {
			p = alloc(data_length);
		
			memcpy(/*dst*/  p, // copy it in
				/*src*/  (void *)data,
				/*len*/  data_length);
			item->data =  (uns8*)p;
		}
			
		item->data_length = data_length;

		return ITEM_QUEUED;
	} else {
		return QUEUE_FULL;
	}	

}	




uns8 its2_router_init(uns16 my_device_id, uns16 network_id) {
	
uns8 lowest_channel_ed;
uns8 count;

//uns8 my_ed[8] = ITS_EA;

	its2_seen_index = 0;

	for (count=0; count < ITS2_SEEN_LIST_SIZE; count++) {
		its2_seen_list[count].its_source_id = 0xffff;
	}	
	
	for (count = 0; count < ITS2_TX_QUEUE_SIZE; count++) {
		its2_tx_queue[count].flag  = ITS2_FLAG_DELETED;
	}
	
	its_init();
	
	its_set_device_id(my_device_id);
	its_set_network_id(network_id);	// network id = controller id = pan id in mode 1
	
	mrf24j40_set_pan_id(network_id);
	mrf24j40_set_short_address(my_device_id); 	// same as device id
	//mrf24j40_set_extended_address(&my_ed);

	debug_str("Router startup\n");
	
	debug_var("Chosing channel", 15);
	debug_nl();	

	// Now set ourselves up on this channel
	mrf24j40_set_channel(15);
	
}	

void its2_request_local_addr(uns16 device_id) {
	its2_router_queue_packet(device_id, ITS_LOCAL_DISCOVER_REQ, NULL, 0, ITS2_FLAG_NO_ACK);
}

void its2_respond_local_addr(uns16 device_id) {

	if ((device_id != 0x0001) || !debug_module) {
		its2_router_queue_packet(device_id, ITS_LOCAL_DISCOVER_RES, NULL, 0, ITS2_FLAG_NO_ACK);
	} else {
		debug_str("IGNORE");
	}	
}

void its2_request_net_addr(uns16 device_id) {
	its2_router_queue_packet(device_id, ITS_NET_DISCOVER_REQ, NULL, 0, ITS2_FLAG_NO_ACK);
}

void its2_respond_net_addr(uns16 device_id) {
	its2_router_queue_packet(device_id, ITS_NET_DISCOVER_RES, NULL, 0, ITS2_FLAG_NO_ACK);
}

void its2_rebroadcast_net_addr_req() {
}


void wpan_data_transmitted_callback(uns8 status, uns8 retries, uns8 channel_busy) {
	its2_transmitting = 0;
}

void its2_router_process() {

	turn_off_mrf_interrupts();

	its2_process_tx_queue();

uns16 test_tick;
	test_tick = tick_get_count();
	if (tick_calc_diff(tick_marker, test_tick) >= state_timeout) {	
	// timeout
	}
	turn_on_mrf_interrupts();
	
}	

its2_result its2_router_queue_packet(uns16 device_id, uns8 packet_type, uns8 *data, uns8 data_length, uns8 ack) {

// return result - queue handle?

uns8 found;
void	*p;
queued_item	*item;
uns8 count;

turn_off_mrf_interrupts();

debug_str("\n[its2_router_queue_packet]\n");

	// find slot
	found = 0;
	for (count = 0; count < ITS2_TX_QUEUE_SIZE; count++) {
		if (its2_tx_queue[count].flag  == ITS2_FLAG_DELETED) {
			found = 1;
			break;
		}
	}
	if (found) {
		debug_var("Qed: ", count);
		item = &its2_tx_queue[count];
		item->flag = ack;	// it's ours!
		item->sent_count = 0;
		
		//item->packet.key_1 = 'I';
		//item->packet.key_2 = 'T';
		// item->length_header; // unknown as yet
		item->packet.packet_type = packet_type;
		item->packet.sequence = its_get_next_sequence();
		item->packet.its_network_id = its_get_network_id();
		item->packet.its_source_id = its_get_device_id();
		item->packet.its_dest_id = device_id;
		if ((packet_type == ITS_LOCAL_DISCOVER_REQ) || 
		    (packet_type == ITS_NET_DISCOVER_REQ)) {
		    item->dest_its_device_id = 0xffff;
		} else {    
			item->dest_its_device_id = device_id;
		}
		item->packet.max_hop_count = ITS2_MAX_HOP_COUNT;
		item->packet.hop_count = 0;
		// item->packet.num_routes;
	
		// uns16 routers[5];
			// create some memory to whack the data in
		if (data_length > 0) {
			p = alloc(data_length);
		
			memcpy(/*dst*/  p, // copy it in
				/*src*/  (void *)data,
				/*len*/  data_length);
			item->data =  (uns8*)p;
		}
			
		item->data_length = data_length;
		
		// Here we will update the route and also set
		// dest_device_handle (which may not be the same as the
		// eventual destination, since we may be going through
		// other hops or doing a broadcast)
		uns8 routing_result = its2_update_route(item);

		turn_on_mrf_interrupts();
		
		if (routing_result == ITS2_UPDATE_ROUTE_FAIL) {
			debug_str("RoUtE fail");
			return ROUTING_TOO_MANY_HOPS;
		} else {
			debug_str("qdone ");
			return ITEM_QUEUED;
		}	
	} else {
		turn_on_mrf_interrupts();
		debug_str(" Full! ");
		return QUEUE_FULL;
	}	
}


void its2_process_tx_queue() {

uns16		current_tick;
uns8		count;
queued_item	*item;
uns8		flag;
uns8		status;
uns16 		my_id;
	//debug_str("\n[PQ]\n");

    current_tick = tick_get_count();
    my_id = its_get_device_id();

	//clear_bit(intcon3, INT1IE);		// disable int1

    
	for (count = 0; count < ITS2_TX_QUEUE_SIZE; count++) { // search for a packet
		item = &its2_tx_queue[count];	// grab item to save code
		flag = item->flag;
		status = item->status;
		if (flag == ITS2_FLAG_DELETED) {
			continue;
		}
		//debug_var("\nQ-", count);
		if (status == QS_WAITING_ON_LOCAL_ADDR) {
			if (item->sent_count == ITS2_SEND_MAX_TRIES) { // Sent too many times
				if (item->packet.its_source_id == my_id) {	// from me?
					debug_str(" Tried local, now time for net search");
					item->sent_count = 0;
					item->status = QS_WAITING_ON_NETWORK_ADDR;
				} else {	// Must have been a network route
					// but we failed to find the next hop locally
					// !! Should return something here to sender
					debug_str("Next hop not available locally. Giving up.");
					its2_transmit_status_callback(&item->packet, ITS2_TX_STATUS_NEXT_HOP_UNKNOWN);
					its2_delete_item_from_queue(item);
				}
			} else if ((item->sent_count == 0) || (tick_calc_diff(item->tick_sent, current_tick)
			         > ITS2_RESEND_TICK_DELAY)) {

			    debug_str(" REQ local address for q ");
			    debug_int(count);
			    debug_str(" dev ");
			    debug_int_hex_16bit(item->dest_its_device_id);

				item->tick_sent = current_tick;
				item->sent_count++;
				its2_request_local_addr(item->dest_its_device_id);
				item->tick_sent = current_tick; // set tick count to current
			} else {
				debug_str(" Local waiting for ");
				debug_int(item->tick_sent);
			}	
		} else
		if (status == QS_WAITING_ON_NETWORK_ADDR) {
			if (item->sent_count == ITS2_SEND_MAX_TRIES) { // Sent too many times
				// !! Need to issue callback here
				debug_str("No addr. killing");
				its2_transmit_status_callback(&item->packet, ITS2_TX_STATUS_NO_ROUTE);
				its2_delete_item_from_queue(item);
			} else {
				//debug_str("Wait for ");
				//debug_int(item->tick_sent);
				//debug_str("+ 5000 current=");
				//debug_int(current_tick);
				if (tick_calc_diff(item->tick_sent, current_tick)
			         > ITS2_RESEND_TICK_DELAY) {
					item->tick_sent = current_tick;
					item->sent_count++;
					its2_request_net_addr(item->dest_its_device_id);
				}
			}
		} else 
		if (status == QS_READY_TO_SEND) {
			if (!its2_transmitting) {
				debug_str("<TX!>");
				its2_transmit(item);
				debug_str("<Done>");
				
			}	
		} else
		if (status == QS_WAITING_ON_ACK) {
			if (item->sent_count > ITS2_SEND_MAX_TRIES) { // Sent too many times
				// !! give up, go home... FAIL! No address
				// issue call back
				// kill from queue
				debug_str("Too many. killing.");
				its2_transmit_status_callback(&item->packet, ITS2_TX_STATUS_NO_ACK);
				its2_delete_item_from_queue(item);
			}
			if (tick_calc_diff(item->tick_sent, current_tick)
			         > ITS2_RESEND_TICK_DELAY) {				
				if (!its2_transmitting) {
					debug_str("<RETRYTX!>");
					its2_transmit(item);
					debug_str("<Done>");
				}
			}	
		} 
		if (status == QS_ACK_RECEIVED) {
			// !! call back here
			its2_transmit_status_callback(&item->packet, ITS2_TX_STATUS_SUCCESS);

			its2_delete_item_from_queue(item);	
		} else
		if (status == QS_SENT) {
			debug_str(" <SENT del> ");
			its2_transmit_status_callback(&item->packet, ITS2_TX_STATUS_SUCCESS);
			its2_delete_item_from_queue(item);
		} else
		if (status == QS_ROUTING_FAILED) {
			debug_str(" <RFAIL del> ");
			its2_transmit_status_callback(&item->packet, ITS2_TX_STATUS_NO_ROUTE);
			its2_delete_item_from_queue(item);
		}
	}
	
	//set_bit(intcon3, INT1IE);		// enable int1

}

void its2_delete_item_from_queue(queued_item *item) {
	if (item->data_length > 0) {
		free((void *)item->data);
	}	
	item->flag = ITS2_FLAG_DELETED;
}
	
//	!!!!! Check if we set dest_deivce_handle to 0xffff when we find a local address
//	set source id to us if we sent it
//	!!!!!
void its2_transmit(queued_item *item) {

uns8 buffer[50];
uns8 count;
uns8 index;
its_device_info *device_info;
	
	its2_transmitting = 1;
	
	debug_str(" X: ");
	its2_print_packet(&item->packet);
	//p = (void *)alloc(item->data_size + ??);
	buffer[0] = 'I';
	buffer[1] = 'T';
	memcpy(/* dst */ (void *)&buffer[3], /* src */ (void *)&item->packet, 11 );
	// now need to copy routes
	index = 11 + 3;
	for (count = 0; count < item->packet.num_routes; count++) {
		buffer[index++] = item->packet.routers[count] & 0xff;
		buffer[index++] = item->packet.routers[count] >> 8;
	}

	buffer[2] = index - 3;	// update header with length
	
	// now copy data
	for (count = 0; count < item->data_length; count++) {
		buffer[index++] = item->data[count];
	}
	debug_str(" To=");
	debug_int_hex_16bit(item->dest_its_device_id);
	debug_str(" bytes=");
	debug_int(index);
	
	item->sent_count++;
	item->tick_sent = tick_get_count();
	if (item->flag == ITS2_FLAG_ACK) {	// ack
		item->status = QS_WAITING_ON_ACK;
	} else {
		item->status = QS_SENT;
	}
	if (item->dest_device_handle != ITS_DEVICE_NONE) { 	// we have handle
		debug_var(" Handle ", item->dest_device_handle);
		device_info = its_get_device_info(item->dest_device_handle);
		if (device_info != NULL) {
			debug_str(" Pan ");
			debug_int_hex_16bit(device_info->addr.local.pan_id);
			debug_str(" SA ");
			debug_int_hex_16bit(device_info->addr.local.short_address);
		} else {
			debug_str("!!Strange!! - NO DEVICE INFO!!\n");
		}	
		mrf24j40_transmit_to_short_address(FRAME_TYPE_DATA, device_info->addr.local.pan_id,
	                                   device_info->addr.local.short_address,
	                                   &buffer, index, MRF_ACK);
	    debug_str(" Sent bytes ");
	    debug_int(index);  
	    debug_putc(' ');                             		
	} 
	else {	// it's a broadcast
		debug_str(" It's a broadcast");
		mrf24j40_transmit_to_short_address(FRAME_TYPE_DATA, 0xffff,
	                                   0xffff, &buffer, index, MRF_NO_ACK);	
	}
		
}


void its2_router_process(queued_item *item) {

}

void its2_print_queue() {

uns8 count;
queued_item *item;

		debug_nl();
		for (count = 0; count < ITS2_TX_QUEUE_SIZE; count++) { 
			item = &its2_tx_queue[count];
			if (item->flag != ITS2_FLAG_DELETED) {
				debug_var("Q:",count);
				debug_var(" FL:", item->flag);
				debug_var(" SRC:", item->packet.its_source_id);
				debug_var(" DST:", item->packet.its_dest_id);
				debug_var(" STAT:", item->status);
				debug_var(" SENT:", item->sent_count);
				debug_var(" TICK:", item->tick_sent);
				debug_nl();
			}
		}
		debug_var("Current tick", tick_get_count());
		debug_nl();		
}
	

// ---------------------------------------------------------------------------
// Device specific routines 
// ---------------------------------------------------------------------------

// void its2_device_receive_callback(uns8 *data, uns8 data_length);

its2_result its2_device_init(uns16 my_device_id, uns16 network_id) {

// ITS_EA is set in config.h

// uns8 my_ea[8] = ITS_EA;

	debug_str("Device startup\n");

	its_init();
	wpan_init();
	
	its_set_device_id(my_device_id);
	its_set_network_id(network_id);	// network id = controller id = pan id in mode 1
	
	mrf24j40_set_pan_id(network_id);
	mrf24j40_set_short_address(my_device_id); 	// same as device id
	//mrf24j40_set_extended_address(&my_ea);

	state = STATE_UNASSOCIATED;
}

its2_result its2_find_controller() {
	state = STATE_SEARCHING;
	tick_marker = tick_get_count();
	channel = 0;
}

void its2_device_process() {

uns16 test_tick;

	// are we searching?
	// has time limit expired?
	// go on to next channel
	// send its association request to everyone
	if (state == STATE_SEARCHING) {
		test_tick = tick_get_count();
		if (tick_calc_diff(tick_marker, test_tick) >= 250) {	// 250 = 1/4 second
			tick_marker = test_tick;
			if (channel == 0) { 
				channel = MRF_FIRST_CHANNEL;
			} else {
				if (channel == MRF_LAST_CHANNEL) {
					// Didn't find the controller
					state = STATE_UNASSOCIATED;
					debug_str("Didn't find controller\n");
				} else {
					channel++;
				}
			}
			// change channel
			debug_var("Trying channel ", channel);
			debug_nl();
			mrf24j40_set_channel(channel);
			// send broadcast association request
			its_transmit_to_sa(/* pan id*/ 0xffff, /* sa */ 0xffff, /* dest device */ 0xffff, 
			                   ITS_ASSOC_REQ, /* nil data */ 0, /* zero length */ 0);
		} // timer for another channel
	} // we're searching	

}


its2_result its2_device_transmit(uns8 *data, uns8 data_length) {

// We want to transmit to our controller
		its_transmit_to_handle(controller_handle,  ITS_GENERIC_DATA, data, data_length);
}


	
