/*

Copyright (c) 2010, Embedded Adventures, www.embeddedadventures.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

- Neither the name of Embedded Adventures nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
THE POSSIBILITY OF SUCH DAMAGE.

Contact us at admin@embeddedadventures.com

*/

/*! \file pic_timer1.h
    \brief Timer 1 support
	
*/

#include "pic_utils.h"

#define TIMER1_PRESCALER_OFF		0b00000000
#define TIMER1_PRESCALER_1_TO_2		0b00010000
#define TIMER1_PRESCALER_1_TO_4		0b00100000
#define TIMER1_PRESCALER_1_TO_8		0b00110000


extern uns16 timer_1_start_value;


/**
	\brief Setup timer 1 with starting values
	
	Turns off timer 1, sets prescaler setting and start value 
	(which will be loaded on each reset).
*/	
void timer_setup_1(uns8 prescaler_setting, uns16 timer_start_value);

/**
	\brief Start timer 1
	
	Kicks off timer 1.
	
*/	
void timer_start_1();

/**
	\brief Stop timer 1
	
	Stops timer 1.
	
*/	
void timer_stop_1();

/** 
	\brief Timer 1 callback function
	
	When a timer 1 interrupt occurs, after handling the interupt and timing issues, this
	callback function is executed. You will need to define this subroutine in your
	code, otherwise linking will fail. 
*/

extern void timer_1_callback();

/**
	\brief handle timer 1 in interrupt service routine
	
	Call this routine in your interrupt subroutine to automatically service timer 1 
	interrupts if they have occursed.
	
*/	

inline void timer_handle_1_isr() {
	uns16 start_value;	
	if (test_bit(pir1, TMR1IF)) {	// interrupt?
		#ifdef _PIC16
			start_value = tmr1l + timer_1_start_value  + 8;	// adjust start value
			tmr1h = start_value >> 8;	// set high value
			tmr1l = start_value & 0xff;	// set low value (must be done in this order)
		#else
			start_value = tmr1l + timer_1_start_value  + 8;	// adjust start value

			tmr1h = start_value >> 8;	// set high value
			tmr1l = start_value & 0xff;	// set low value (must be done in this order)
		#endif
		timer_1_callback();	// call the callback
		clear_bit( pir1, TMR1IF ); //clear timer 0 interrupt bit
	}
}	
