/*

Copyright (c) 2010, Embedded Adventures, www.embeddedadventures.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

- Neither the name of Embedded Adventures nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
THE POSSIBILITY OF SUCH DAMAGE.

Contact us at admin@embeddedadventures.com

*/


#include "sht15.h"
#include "pic_utils.h"
#include "pic_serial.h"

#define sht15_write_sda() 	clear_bit(tris_array[sht15_sda_port - PORTA], sht15_sda_pin);   // output
#define sht15_read_sda()  set_bit(tris_array[sht15_sda_port - PORTA], sht15_sda_pin);   // input


#define CHECK_TEMP 0b00000011
#define CHECK_HUMD 0b00000101
#define CHECK_STAT 0b00000111
#define WRITE_STAT 0b00000110


void sht15_setup_io(void)
{
	clear_pin(sht15_sck_port, sht15_sck_pin);
	clear_pin(sht15_sda_pin, sht15_sda_pin);
	make_output(sht15_sda_port, sht15_sda_pin);
	make_output(sht15_sck_port, sht15_sck_pin);
}

uns16 sht15_read_humidity(void) {

    uns16 response;

    sht15_start();
    sht15_send_byte(CHECK_HUMD);
    response = sht15_read_byte16();
    response = response >> 4;
	response = sht15_fix_humidity_l(response);
	return response;
}

uns16 sht15_read_temperature(void) {

uns16 response;
	
	sht15_start();
    sht15_send_byte(CHECK_TEMP);
    response = sht15_read_byte16(); //Listen for response from SHT15
    serial_print_str("\nT=");
    serial_print_int(response);
	response = sht15_fix_temperature_h(response);
    serial_print_str("\nT=");
    serial_print_int(response);

}	

//Init the sensor and read out the humidity and temperature data
void sht15_read(void)
{
    uns16 response;

    //Issue command start
    sht15_start();

    //Now send command code
    sht15_send_byte(CHECK_HUMD);
    
    response = sht15_read_byte16();
    
    
    serial_print_str("\nH=");
    serial_print_int(response);
    //response = response >> 4;
	response = sht15_fix_humidity(response);
    serial_print_str("\nH=");
    serial_print_int(response);

    sht15_start();
    sht15_send_byte(CHECK_TEMP);
    response = sht15_read_byte16(); //Listen for response from SHT15
    serial_print_str("\nT=");
    serial_print_int(response);//
	response = sht15_fix_temperature_h(response);
    serial_print_str("\nT=");
    serial_print_int(response);

}

void sht15_send_byte(uns8 sht15_command)
{
    uns8 i;


    sht15_write_sda();


    clear_pin(sht15_sck_port, sht15_sck_pin);
    for(i = 0 ; i < 8 ; i++)
    {
		delay_us(10);
        change_pin(sht15_sda_port, sht15_sda_pin, sht15_command.7);
        sht15_command = sht15_command << 1;
		delay_us(10);
        set_pin(sht15_sck_port, sht15_sck_pin);
		delay_us(10);
		clear_pin(sht15_sck_port, sht15_sck_pin);
    }
	delay_us(100);
    //Wait for SHT15 to acknowledge.
//    clear_pin(sht15_sck_port, sht15_sck_pin);
    sht15_read_sda();
    // <<!!>> Fix following
//    while (test_pin(sht15_sda_port, sht15_sda_pin) == 1); //Wait for SHT to pull line low
    
    set_pin(sht15_sck_port, sht15_sck_pin);
    // read ack here
    
    delay_us(10);
    clear_pin(sht15_sck_port, sht15_sck_pin); //Falling edge of 9th clock
    
    while (test_pin(sht15_sda_port, sht15_sda_pin) == 0); //Wait for SHT to release line

    //Wait for SHT15 to pull SDA low to signal measurement completion. 
    //This can take up to 210ms for 14 bit measurements
    i = 0;
    while ((test_pin(sht15_sda_port, sht15_sda_pin) == 1)) //Wait for SHT to pull line low
    {
        i++;
        if (i == 255) break;
        
        delay_ms(10);
    }

    //Debug info
    i *= 10; //Convert to ms
    serial_print_str("\nRt=");
    serial_print_int(i);
    serial_print_str("ms\n"); //Debug to see how long response took

}

//Specific SHT start command
void sht15_start(void)
{
/*    sht15_write_sda();
    set_pin(sht15_sda_port, sht15_sda_pin); 	//SDA = 1;
    set_pin(sht15_sck_port, sht15_sck_pin); //SCK = 1;
    
    clear_pin(sht15_sda_port, sht15_sda_pin);	//SDA = 0;
    clear_pin(sht15_sck_port, sht15_sck_pin); //SCK = 0;
    set_pin(sht15_sck_port, sht15_sck_pin); // SCK = 1;
    set_pin(sht15_sda_port, sht15_sda_pin); 	//SDA = 1;
    clear_pin(sht15_sck_port, sht15_sck_pin); //SCK = 0;
*/
    sht15_write_sda();
	
	// initial state
	
    clear_pin(sht15_sck_port, sht15_sck_pin); //SCK = 0;
    set_pin(sht15_sda_port, sht15_sda_pin); 	//SDA = 1;
		delay_us(10);

    set_pin(sht15_sck_port, sht15_sck_pin); // SCK = 1;
		delay_us(10);

	// sck is high, so lower the data ling
    clear_pin(sht15_sda_port, sht15_sda_pin);	//SDA = 0;
		delay_us(10);
		
	// low pulse SCK	
    clear_pin(sht15_sck_port, sht15_sck_pin); //SCK = 0;
		delay_us(10);
    set_pin(sht15_sck_port, sht15_sck_pin); // SCK = 1;
		delay_us(10);
		
	// now raise SDA while SCK is high
    set_pin(sht15_sda_port, sht15_sda_pin); 	//SDA = 1;
		delay_us(10);
		
	// return sck to normal state
    clear_pin(sht15_sck_port, sht15_sck_pin); //SCK = 0;
		delay_us(10);

}

//Read 16 bits from the SHT sensor
uns16 sht15_read_byte16(void)
{
    uns8 j;
    uns16 in_byte;
	uns8 crc;
	//bit  my_store_gie = intcon.GIE;        
    //kill_interrupts();

    clear_pin(sht15_sck_port, sht15_sck_pin);

    sht15_read_sda();

    for(j = 0 ; j < 8 ; j++)
    {
        set_pin(sht15_sck_port, sht15_sck_pin);
delay_us(10);
        in_byte = in_byte << 1;
        in_byte.0 = test_pin(sht15_sda_port, sht15_sda_pin);
		
        clear_pin(sht15_sck_port, sht15_sck_pin);
delay_us(10);
    }
    delay_us(100);
    
	// send ack
	sht15_write_sda();
    clear_pin(sht15_sda_port, sht15_sda_pin);
    
        set_pin(sht15_sck_port, sht15_sck_pin);
delay_us(10);
        clear_pin(sht15_sck_port, sht15_sck_pin);
delay_us(10);
    
    sht15_read_sda();
    delay_us(100);
    
    
    for(j = 0 ; j < 8 ; j++)
    {
        set_pin(sht15_sck_port, sht15_sck_pin);
delay_us(10);
        in_byte = in_byte << 1;
        in_byte.0 = test_pin(sht15_sda_port, sht15_sda_pin);

        clear_pin(sht15_sck_port, sht15_sck_pin);
delay_us(10);
    }

	// send ack
	sht15_write_sda();
    clear_pin(sht15_sda_port, sht15_sda_pin);
    
        set_pin(sht15_sck_port, sht15_sck_pin);
delay_us(10);
        clear_pin(sht15_sck_port, sht15_sck_pin);
delay_us(10);
    
    sht15_read_sda();
    delay_us(100);
    
    
    for(j = 0 ; j < 8 ; j++)
    {
        set_pin(sht15_sck_port, sht15_sck_pin);
delay_us(10);
        crc = crc << 1;
        crc.0 = test_pin(sht15_sda_port, sht15_sda_pin);

        clear_pin(sht15_sck_port, sht15_sck_pin);
delay_us(10);
    }
    
	// send ack
	sht15_write_sda();
    clear_pin(sht15_sda_port, sht15_sda_pin);
    
        set_pin(sht15_sck_port, sht15_sck_pin);
delay_us(10);
        clear_pin(sht15_sck_port, sht15_sck_pin);
delay_us(10);
    
    sht15_read_sda();
    delay_us(100);
     
	//intcon.GIE = my_store_gie;
    return(in_byte);
}


uns16 sht15_fix_humidity_r(uns16 sensor_out) {
	
int32 c1 = -40000000;
int32 c2 = 405000;
int32 c3 = 28;
int32 s;
int32 final;


s = sensor_out;

s = s * 10000000;

final = c1 + c2 * s + c3 * s * s;

final = final / 10000;

return final;
} 
	


uns16 sht15_fix_humidity(uns16 sensor_out)
{
	
long result; // 16Bit unsigned for the result

	if ( sensor_out <= 1712 ) {
		result = 143 * sensor_out;
		result = result - 8192; // result = a * sensor_out
		//result < 512 ? result = 512; // check for underflow
		//result = result  8192; // result = result + b
	} else {
		result = 111 *sensor_out + 46288; // result = a * sensor_out
		//result = result + 46288; // result = result + b
		//result > 25600 ? result = 25600; // check for overflow (optional)
	}
	//8 MSBs are 0-100%RH integers, 8 LSBs are remainder
	result = result / 4096;
	return result;
}

uns16 sht15_fix_humidity_l(uns8 sensor_out)
{

uns16 result; // 16Bit unsigned for the result

	if ( sensor_out <= 107 ) {
		result = 143 * sensor_out; // result = a * sensor_out
		if (result < 512) { result = 512; } // check for underflow
		result -= 512; 
	} else {
		result = 111 * sensor_out; // result = a * sensor_out
		result += 2893; // result = result + b
		if (result > 25600) { result = 25600; } // check for overflow (optional)
	}
	//8 MSBs are 0-100%RH integers, 8 LSBs are remainder
	result = result >> 8; // result = result / 256
	return result;
}

int16 sht15_fix_temperature_h(uns16 sensor_out)
{
	int16 result;
	
//	result = -3963 + sensor_out;
  	result = -4001 + sensor_out;
    return result;
}