/*

Copyright (c) 2010, Embedded Adventures, www.embeddedadventures.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

- Neither the name of Embedded Adventures nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
THE POSSIBILITY OF SUCH DAMAGE.

Contact us at admin@embeddedadventures.com

*/

/** \file spi.h
	\brief Outputs SPI-like interfaces (clock+data)
	
	
	Covers standard SPI-like interfaces (clock + data) and
	Sure Electronics displays which are a little different
*/	

#ifndef __spi_h
#define __spi_h

#include "pic_utils.h"
#include "config.h"

// Include the following in your config.h:
/*
// - - - - - - - - - - - 
// spi defines
// - - - - - - - - - - -

#define spi_clk_port PORTA
#define spi_data_port PORTA
#define spi_clk_pin  2
#define spi_data_pin  3

// - - - - - - - - - - -
*/



/**
	\brief Send a byte of data using software spi
	
	Sends a byte of data MSB first, data only changes
	on clock low
*/
void spi_write(uns8 data);

/**
	\brief Send a byte of data using software spi
	
	Sends a byte of data LSB first, data only changes
	on clock low
*/
void spi_write_lsb(uns8 data);

/**
	\brief Setup ports and pins for SPI output
	
	Setup ports and pins for SPI output
*/
void spi_setup();

/**
	\brief SPI write for Sure devices
	
	SPI write byte for Sure devices.  Sure devices
	do things a little differently. Data goes LSB first but data
	changes on clock high.
*/
void spi_write_sure(uns8 data);

/** 
	\brief SPI test routine
	
*/
void spi_pulse_0();
/** 
	\brief SPI test routine
	
*/
void spi_pulse_1();


#ifndef spi_clk_port
	#error "spi_clk_port not defined in config.h"
#endif	
#ifndef spi_data_port
	#error "spi_data_port not defined in config.h"
#endif	
#ifndef spi_clk_pin
	#error "spi_clk_pin not defined in config.h"
#endif	
#ifndef spi_data_pin
	#error "spi_data_pin not defined in config.h"
#endif	



#endif