//-----------------------------------------------------
// Pic Pack library
// 
// ea_plt1003_usb2serial.c
//
// For use with the Embedded Adventures USB2Serial module
// PLT-1003
//
//
//-----------------------------------------------------


// Pic Pack includes
#include "config.h"
#include "pic_utils.h"
#include "pic_usb.h"

#include "platform_leds.h"
#include "pic_timer.h"
#include "pic_tick.h"
#include "usb_cdc_class.h"

// System includes
#include "string.h"
#include "stdlib.h"


// If you'd like USB debug info out the serial connection, uncomment this line:
//#define USB_SERIAL_DEBUG
// You also might like to uncomment the USB_DEBUG and CDC_DEBUG lines in the config.h


#include "pic_serial.h"

// Platform LEDs
#include "platform_leds.h"

// Put received characters in here to see
// if they make a command
char serial_buffer[6]= '\0';

// See if we need to do a task
uns8 do_task = 0;

// Flag if we received something
bit receive = 0;

// Dummy variable to show we can set it 
uns8 send_to = 0;


// a check to see if the usb connection has been configured yet
uns8 usb_configured = 0;

	
void interrupt() {
	
	usb_handle_isr();
	// Handle timer 0 interrupt
	timer_handle_0_isr();	

	serial_handle_tx_isr();
	serial_handle_rx_isr();

}


void usb_device_configured_callback() {
	usb_configured = 1;
}	

// Process_rx
//
// Keep adding characters to our input buffer
// when we enter a command, then act on it

void process_usb_rx() {
	uns8 rec = usb_cdc_getc();	// get the character from the fifo
	serial_putc(rec);
}

void process_serial_rx() {
	
	uns8 rec = serial_getc();	// get the character from the fifo
	usb_cdc_putc(rec);
}


// configure_system
//
// Get the pic ready for everything we want to do


void configure_system() {
	    
	kill_interrupts();	// turn off interrupts just in case

	turn_analog_inputs_off();	// kill those pesky analogue inputs

	platform_leds_setup_io();
	
	//#ifdef USB_DEBUG
	serial_setup(SPBRG_115200);
	//#endif
	usb_cdc_setup();

	usb_setup();
	
	// setup i/o for platform LEDs

	platform_leds_setup_io();

	// Timer 0 is used for ticks - 1ms at 48mhz (12mips)
	timer_setup_0(TIMER_16BIT_MODE, TIMER_PRESCALER_OFF, 0xffff - 12000 -1); 
	
	turn_usb_ints_on();
	turn_peripheral_ints_on();
	turn_global_ints_on();
}




void main() {

bit	usb_flag = 1;
		
	configure_system();
	
	timer_start_0();
	
	// Enable LED flashing
	
	platform_leds_flashing(1, 1);
	platform_leds_flashing(2, 1);
	platform_leds_flashing(3, 1);

	// Let the flashing begin (delay also gives everything a chance to settle)
	
	platform_leds_flash(1);
	delay_ms(50);
	platform_leds_flash(2);
	delay_ms(50);
	platform_leds_flash(3);
	
	// Tell USB subsystem to do its thing
	
	usb_enable_module();

	
	// Main loop
	for (;;) {

		if (usb_flag) {
			if (usb_configured) {
				// Okay, we're now configured. Stop the flashing
				usb_flag = 0;
				platform_leds_flashing(1, 0);
				platform_leds_flashing(2, 0);
				platform_leds_flashing(3, 0);
			}
		}		
	
		if (usb_configured) {
			if (usb_cdc_rx_avail()) {
				process_usb_rx();
			}
			if (serial_rx_avail()) {
				process_serial_rx();
			}
		}

		start_crit_sec();
			platform_leds_process();
		end_crit_sec();
	}	// while (1)
	
	// main
}